﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using VillainmadEngine;

namespace Villainmad.Stages
{
    class TestStage : Stage
    {
        #region Values
        public override int PlayerLayer { get { return 9; } }

        public override Rectangle GameArea
        {
            get { return new Rectangle(128, 16, 384, 448); }
        }

        public override Rectangle PlayerField
        {
            get { return new Rectangle(20, 20, 344, 408); }
        }

        public override int NumLayers { get { return 16; } }

        public override Vector2 PlayerStart
        {
            get { return new Vector2(192, 384); }
        }

        public override string[] SoundEffectList
        {
            get
            {
                return new string[] {
                    "Tiptap",
                    "BGM\\loopytesty"
                };
            }
        }

        public override string[] Texture2DList
        {
            get
            {
                return new string[]
                {
                    "Sprites/bulOrb", 
                    "Sprites/bulOrbSheet",
                    "Background/Fog", 
                    "Sprites/whiteSquare",
                    "Sprites/LexyLeft2", 
                    "Sprites/LexyLeft1",
                    "Sprites/LexyNeutral",
                    "Sprites/LexyRight1",
                    "Sprites/LexyRight2",
                    "Sprites/hitbox"
                };
            }
        }
        #endregion

        private Vector2 StartCenter = new Vector2(192, 128);
        
        #region Constructor
        public TestStage()
        {
        }
        #endregion

        #region Rotate
        float Rot
        {
            get { return _rot; }
            set { _rot = value % 360; }
        }
        float _rot = 0f;

        public void Rotate()
        {
            AddEntity<EntityBall>(StartCenter, Utilities.FromAngleDeg(Rot, 0.4f));
            
            AddEntity<EntityBall>(StartCenter, Utilities.FromAngleDeg(Rot + 180, 0.4f));
            
            Rot += 5.6f;

            EnqueueCommand(3, delegate() { Rotate(); });
        }

        public void Shenanigans(GameEntity gameEntity)
        {
            if (!gameEntity.Extant) return;

            gameEntity.MoveToSecs(new Vector2(128, 192), 1);
            EnqueueCommandSecs(1.5f, delegate()
            {
                gameEntity.Velocity = new Vector2(2, -2);
            });
        }
        #endregion

        #region Overrides
        const string BgImage = "Background/Fog";
        public override void DrawBackground(GameTime gameTime)
        {
            Vector2 tor = Vector2.Zero;
            for (int y = 0; y <= 3; y++)
            {
                //I have no idea why this works, it just does.
                tor.Y = (y * 64 + (Frame % 64)) * 4 - 256;
                tor.X = 0;
                Drawing.Draw(BgImage, tor, Color.White);
                tor.X = 256;
                Drawing.Draw(BgImage, tor, Color.White);
            }
        }

        public override void OnEnter()
        {
            Sound.BGMStart("BGM\\loopytesty");
            Rotate();
            
            GameEntity Blah = AddEntity<SquareThing>(new Vector2(0f, 128f),
                new Vector2(0.3f, 0.1f));

            EnqueueCommandSecs(2, delegate() { Shenanigans(Blah); });

            AddPlayer<PlayerLexy>();
        }

        protected override void Input(GameInput InputList, 
            GameInput InputPressed, GameInput InputReleased)
        {
            if ((InputPressed & GameInput.Esc) != 0)
                Menu.StartMenu<PauseMenu>();
        }
        #endregion
    }
}

