﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace VillainmadEngine
{
    public partial struct Animation
    {

        #region Step, Vector2, Color
        /// <summary>
        /// Draws the current frame of animation.
        /// </summary>
        /// <param name="Step">Number of steps since the animation began.</param>
        /// <param name="Pos">Position to draw the animation-frame.</param>
        /// <param name="color">Color to tint the texture.</param>
        public void Draw(int Step, Vector2 Pos, Color color)
        {
            int curFrame = CurFrame(Step);
            Drawing.Draw(Texture, Pos, SourceRects[curFrame], color, 0F,
                Origins[curFrame], 1F, SpriteEffects.None, Drawing.DefaultBlend);
        }
        #endregion

        #region Step Pos Color Rot Effects Blend
        /// <summary>
        /// Draws the current frame of animation.
        /// </summary>
        /// <param name="Step">Number of steps since the animation began.</param>
        /// <param name="Pos">Position to draw the animation-frame.</param>
        /// <param name="color">Color to tint the texture.</param>
        /// <param name="rotation">Specifies the angle (in radians) to rotate
        /// the frame around its center.</param>
        /// <param name="spriteEffects">Effects to apply.</param>
        /// <param name="blendState">BlendState to apply. Default is 
        /// Drawing.DefaultBlend (BlendState.AlphaBlend).</param>
        public void Draw(int Step, Vector2 Pos, Color color, float rotation,
            SpriteEffects spriteEffects, BlendState blendState)
        {
            int curFrame = CurFrame(Step);
            Drawing.Draw(Texture, Pos, SourceRects[curFrame], color, rotation,
                Origins[curFrame], 1F, spriteEffects, blendState);
        }
        #endregion

        #region Step Pos Color Rot Float-Scale Effects Blend
        /// <summary>
        /// Draws the current frame of animation.
        /// </summary>
        /// <param name="Step">Number of steps since the animation began.</param>
        /// <param name="Pos">Position to draw the animation-frame.</param>
        /// <param name="color">Color to tint the texture.</param>
        /// <param name="rotation">Specifies the angle (in radians) to rotate
        /// the sprite around its center.</param>
        /// <param name="scale">Scale factor; 1F = full size.</param>
        /// <param name="spriteEffects">Effects to apply.</param>
        /// <param name="blendState">BlendState to apply. Default is 
        /// Drawing.DefaultBlend (BlendState.AlphaBlend).</param>
        public void Draw(int Step, Vector2 Pos, Color color, float rotation,
            float scale, SpriteEffects spriteEffects, BlendState blendState)
        {
            int curFrame = CurFrame(Step);
            Drawing.Draw(Texture, Pos, SourceRects[curFrame], color, rotation,
                Origins[curFrame], scale, spriteEffects, blendState);
        }
        #endregion

        #region Step Pos Color Rot Vector2-Scale Effects Blend
        /// <summary>
        /// Draws the current frame of animation.
        /// </summary>
        /// <param name="Step">Number of steps since the animation began.</param>
        /// <param name="Pos">Position to draw the animation-frame.</param>
        /// <param name="color">Color to tint the texture.</param>
        /// <param name="rotation">Specifies the angle (in radians) to rotate
        /// the sprite around its center.</param>
        /// <param name="scale">Scale factor; put 1F for full size.</param>
        /// <param name="spriteEffects">Effects to apply.</param>
        /// <param name="blendState">BlendState to apply. Default is 
        /// Drawing.DefaultBlend (BlendState.AlphaBlend).</param>
        public void Draw(int Step, Vector2 Pos, Color color, float rotation,
            Vector2 scale, SpriteEffects spriteEffects, BlendState blendState)
        {
            int curFrame = CurFrame(Step);
            Drawing.Draw(Texture, Pos, SourceRects[curFrame], color, rotation,
                Origins[curFrame], scale, spriteEffects, blendState);
        }
        #endregion
    }
}
