﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using VillainmadEngine.Configuration;

namespace VillainmadEngine
{
    /// <summary>
    /// The base type ... for VILLAINMAD.
    /// </summary>
    public sealed partial class Basis : Microsoft.Xna.Framework.Game
    {
        #region Fields
        private GraphicsDeviceManager graphics;
        
        /// <summary>
        /// True if the game has started; false otherwise.
        /// </summary>
        public bool gameStarted
        {
            get { return (DataStealer.StateStack.Count != 0); }
        }
        #endregion

        #region Constructor
        /// <summary>
        /// Constructs a new instance of this. DON'T USE THIS.
        /// </summary>
        private Basis(Type Default)
        {
            DataStealer.DefaultType = Default;
            Config.Load();

            DataStealer.curRes = ScreenResolution.Resolutions[Config.ScreenRes];

            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = DataStealer.curRes.ResWidth;


            graphics.PreferredBackBufferHeight = DataStealer.curRes.ResHeight;

            graphics.IsFullScreen = DataStealer.curRes.FullScreen;

            DataStealer.Basis = this;

            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Creates a new basis. Only use this in Program.cs!
        /// </summary>
        /// <typeparam name="type">The type of the new basis.</typeparam>
        /// <returns></returns>
        public static Basis Create<type>()
            where type : GameState, new()
        {
            return new Basis(typeof(type));
        }
        #endregion

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override sealed void Initialize()
        {
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override sealed void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            DataStealer.SpriteBatch = new SpriteBatch(GraphicsDevice);
            //Load fonts.

            DataStealer.DefaultFont = Content.Load<SpriteFont>("lang/Arbitrary");

            DataStealer.SpriteScale = Matrix.CreateScale(DataStealer.curRes.Multiplier,
                DataStealer.curRes.Multiplier, 1f);

            DataStealer.DefaultViewport = new Viewport(DataStealer.curRes.ScreenRect);
            GraphicsDevice.Viewport = DataStealer.DefaultViewport;
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override sealed void UnloadContent()
        {
        }

        /// <summary>
        /// Called after all components are initialized but before the
        /// first update in the game loop.
        /// </summary>
        protected override void BeginRun()
        {
            base.BeginRun();

            GameState.LoadState();
        }

        /// <summary>
        /// Exits the game.
        /// </summary>
        public new static void Exit()
        {
            Game game = DataStealer.Basis;
            game.Exit();
        }
    }
}