﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;
using System.Configuration;
using Microsoft.Xna.Framework.Input;

namespace VillainmadEngine.Configuration
{
    internal static class Config
    {
        #region Default
        /// <summary>
        /// Defaults values.
        /// </summary>
        public static class Default
        {
            #region Buttons
            /// <summary>
            /// "Enter" button.
            /// </summary>
            public static int ButEnter { get { return 3; } }

            /// <summary>
            /// "Esc" button.
            /// </summary>
            public static int ButEsc { get { return 6; } }

            /// <summary>
            /// "Fire 1" button.
            /// </summary>
            public static int ButFire1 { get { return 0; } }

            /// <summary>
            /// "Fire 2" button.
            /// </summary>
            public static int ButFire2 { get { return 1; } }

            /// <summary>
            /// "Fire 3" button.
            /// </summary>
            public static int ButFire3 { get { return 2; } }

            /// <summary>
            /// "Focus" button.
            /// </summary>
            public static int ButFocus { get { return 4; } }

            /// <summary>
            /// "Skip" button.
            /// </summary>
            public static int ButSkip { get { return 5; } }
            #endregion
            /// <summary>
            /// Controller deadzone.
            /// </summary>
            public static float Deadzone { get { return 0.25f; } }

            #region Keys
            /// <summary>
            /// "Enter" key.
            /// </summary>
            public static Keys KeyEnter { get { return Keys.Enter; } }

            /// <summary>
            /// "Esc" key.
            /// </summary>
            public static Keys KeyEsc { get { return Keys.Escape; } }

            /// <summary>
            /// "Fire1" key.
            /// </summary>
            public static Keys KeyFire1 { get { return Keys.Z; } }

            /// <summary>
            /// "Fire2" key.
            /// </summary>
            public static Keys KeyFire2 { get { return Keys.X; } }

            /// <summary>
            /// "Fire3" key.
            /// </summary>
            public static Keys KeyFire3 { get { return Keys.C; } }

            /// <summary>
            /// "Focus" key.
            /// </summary>
            public static Keys KeyFocus { get { return Keys.LeftShift; } }

            /// <summary>
            /// "Skip" key.
            /// </summary>
            public static Keys KeySkip { get { return Keys.LeftControl; } }
            #endregion
            /// <summary>
            /// Screen resolution multiplier.
            /// </summary>
            public static byte ScreenRes { get { return 2; } }

            /// <summary>
            /// Fullscreen.
            /// </summary>
            public static bool ScreenFull { get { return false; } }

            /// <summary>
            /// Music volume.
            /// </summary>
            public static float VolMusic { get { return 1f; } }

            /// <summary>
            /// Sound volume.
            /// </summary>
            public static float VolSound { get { return 1.0f; } }
        }
        #endregion

        #region Values
        private static Dictionary<string, Keys> _keyList = 
            new Dictionary<string, Keys>
        {
            { "Enter", Default.KeyEnter },
            { "Esc", Default.KeyEsc },
            { "Fire1", Default.KeyFire1 },
            { "Fire2", Default.KeyFire2 },
            { "Fire3", Default.KeyFire3 },
            { "Focus", Default.KeyFocus },
            { "Skip", Default.KeySkip }
        };

        private static Dictionary<string, int> _butList = 
            new Dictionary<string, int>
        {
            { "Enter", Default.ButEnter },
            { "Esc", Default.ButEsc },
            { "Fire1", Default.ButFire1 },
            { "Fire2", Default.ButFire2 },
            { "Fire3", Default.ButFire3 },
            { "Focus", Default.ButFocus },
            { "Skip", Default.ButSkip }
        };

        private static float _deadZone = Default.Deadzone;

        private static bool _screenFull = Default.ScreenFull;

        private static int _screenRes = Default.ScreenRes;

        private static float _volMusic = Default.VolMusic;

        private static float _volSound = Default.VolSound;

        /// <summary>
        /// "Enter" button.
        /// </summary>
        public static int ButEnter
        {
            get { return _butList["Enter"]; }
            set { _butList["Enter"] = value; }
        }

        /// <summary>
        /// "Esc" button.
        /// </summary>
        public static int ButEsc
        {
            get { return _butList["Esc"]; }
            set { _butList["Esc"] = value; }
        }

        /// <summary>
        /// "Fire1" button.
        /// </summary>
        public static int ButFire1
        {
            get { return _butList["Fire1"]; }
            set { _butList["Fire1"] = value; }
        }

        /// <summary>
        /// "Fire2" button.
        /// </summary>
        public static int ButFire2
        {
            get { return _butList["Fire2"]; }
            set { _butList["Fire2"] = value; }
        }

        /// <summary>
        /// "Fire3" button.
        /// </summary>
        public static int ButFire3
        {
            get { return _butList["Fire3"]; }
            set { _butList["Fire3"] = value; }
        }

        /// <summary>
        /// "Focus" button.
        /// </summary>
        public static int ButFocus
        {
            get { return _butList["Focus"]; }
            set { _butList["Focus"] = value; }
        }

        /// <summary>
        /// "Skip" button.
        /// </summary>
        public static int ButSkip
        {
            get { return _butList["Skip"]; }
            set { _butList["Skip"] = value; }
        }

        /// <summary>
        /// "Enter" key.
        /// </summary>
        public static Keys KeyEnter
        {
            get { return _keyList["Enter"]; }
            set { _keyList["Enter"] = value; }
        }

        /// <summary>
        /// "Esc" key.
        /// </summary>
        public static Keys KeyEsc
        {
            get { return _keyList["Esc"]; }
            set { _keyList["Esc"] = value; }
        }

        /// <summary>
        /// "Fire1" key.
        /// </summary>
        public static Keys KeyFire1
        {
            get { return _keyList["Fire1"]; }
            set { _keyList["Fire1"] = value; }
        }

        /// <summary>
        /// "Fire2" key.
        /// </summary>
        public static Keys KeyFire2
        {
            get { return _keyList["Fire2"]; }
            set { _keyList["Fire2"] = value; }
        }

        /// <summary>
        /// "Fire3" key.
        /// </summary>
        public static Keys KeyFire3
        {
            get { return _keyList["Fire3"]; }
            set { _keyList["Fire3"] = value; }
        }

        /// <summary>
        /// "Focus" key.
        /// </summary>
        public static Keys KeyFocus
        {
            get { return _keyList["Focus"]; }
            set { _keyList["Focus"] = value; }
        }

        /// <summary>
        /// "Skip" key.
        /// </summary>
        public static Keys KeySkip
        {
            get { return _keyList["Skip"]; }
            set { _keyList["Skip"] = value; }
        }

        /// <summary>
        /// Controller thumbstick deadzone.
        /// </summary>
        public static float Deadzone
        {
            get { return _deadZone; }
            set { _deadZone = value; }
        }

        /// <summary>
        /// Fullscreen or not.
        /// </summary>
        public static bool ScreenFull
        {
            get { return _screenFull; }
            set { _screenFull = value; }
        }

        /// <summary>
        /// Screen resolution multiplier.
        /// </summary>
        public static int ScreenRes
        {
            get { return _screenRes; }
            set { _screenRes = value; }
        }

        /// <summary>
        /// Music volume.
        /// </summary>
        public static float VolMusic
        {
            get { return _volMusic; }
            set { _volMusic = value; }
        }

        /// <summary>
        /// Sound volume.
        /// </summary>
        public static float VolSound
        {
            get { return _volSound; }
            set { _volSound = value; }
        }
        #endregion

        private static string SettingsFile =
#if DEBUG
            "settings.xml";
//*
#else
            Path.Combine(Environment.GetFolderPath(
                Environment.SpecialFolder.ApplicationData),
                "NoWorryNeverfear",
                "Villainmad",
                "settings.xml");

        private void CheckDir()
        {
            string path = Path.Combine(Environment.GetFolderPath(
                Environment.SpecialFolder.ApplicationData),
                "NoWorryNeverfear");
            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            path = Path.Combine(path, "Villainmad");

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);
        }
#endif//*/
        public static void Load()
        {
            XmlTextReader textReader = new XmlTextReader(SettingsFile);

            try
            {
                Dictionary<string, string> optList = new Dictionary<string, string>();

                while (textReader.Read())
                {
                    if (textReader.NodeType != XmlNodeType.Element)
                        continue;
                    if (textReader.Depth == 0)
                    {
                        if (textReader.Name == "settings") continue;
                        else throw new InvalidDataException("Invalid Data: "
                            + textReader.Name);
                    }
                    if (textReader.Depth != 0 && textReader.Depth != 1)
                        throw new InvalidDataException("Data-depth level too deep!");
                    string name = textReader.Name;

                    optList.Add(name, textReader.ReadElementContentAsString());
                }

                textReader.Close();

                string[] keyList = _keyList.Keys.ToArray<string>();

                foreach (string curKey in keyList)
                {
                    string curVal = optList["Key" + curKey];

                    _keyList[curKey] =
                        (Keys)Enum.Parse(typeof(Keys),
                        curVal, true);
                }

                keyList = _butList.Keys.ToArray<string>();

                foreach (string curKey in keyList)
                {
                    string curVal = optList["But" + curKey];
                    _butList[curKey] = Convert.ToInt32(curVal);
                }
                _deadZone = Convert.ToSingle(optList["Deadzone"]);
                _screenRes = Convert.ToByte(optList["ScreenRes"]);
                _screenFull = Convert.ToBoolean(optList["ScreenFull"]);
            }
            catch (Exception e)
            {
                textReader.Close();
#if DEBUG
                System.Windows.Forms.MessageBox.Show(e.ToString());
#else
                e.ToString();
#endif
                Reset();
                Save();
            }
            KbiReset();
        }

        public static void Save()
        {
#if !DEBUG
            CheckDir();
#endif

            XmlWriterSettings xSettings = new XmlWriterSettings
            {
                Encoding = Encoding.UTF8,
                Indent = true
            };
            XmlWriter textWriter = XmlWriter.Create(SettingsFile, xSettings);

            textWriter.WriteStartDocument();
            textWriter.WriteStartElement("settings");

            foreach (KeyValuePair<string, Keys> curPair in _keyList)
            {
                textWriter.WriteElementString("Key" + curPair.Key,
                    curPair.Value.ToString());
            }

            foreach (KeyValuePair<string, int> curPair in _butList)
            {
                textWriter.WriteElementString("But" + curPair.Key, 
                    curPair.Value.ToString());
            }

            textWriter.WriteElementString("ScreenRes", ScreenRes.ToString());

            textWriter.WriteElementString("ScreenFull", 
                ScreenFull ? "true" : "false");

            textWriter.WriteElementString("Deadzone", Deadzone.ToString());

            textWriter.WriteEndDocument();
            textWriter.Close();
        }

        public static void Reset()
        {
            ButEnter = Default.ButEnter;
            ButEsc = Default.ButEsc;
            ButFire1 = Default.ButFire1;
            ButFire2 = Default.ButFire2;
            ButFire3 = Default.ButFire3;
            ButFocus = Default.ButFocus;
            ButSkip = Default.ButSkip;
            Deadzone = Default.Deadzone;
            KeyEnter = Default.KeyEnter;
            KeyEsc = Default.KeyEsc;
            KeyFire1 = Default.KeyFire1;
            KeyFire2 = Default.KeyFire2;
            KeyFire3 = Default.KeyFire3;
            KeyFocus = Default.KeyFocus;
            KeySkip = Default.KeySkip;
            ScreenFull = Default.ScreenFull;
            ScreenRes = Default.ScreenRes;
            VolMusic = Default.VolMusic;
            VolSound = Default.VolSound;
            KbiReset();
        }

        private static void KbiReset()
        {
            Microsoft.Xna.Framework.Audio.SoundEffect.MasterVolume =
                VolSound;
            _kbiList = new KBI[] {
                new KBI() { key = KeyEnter, button = ButEnter,
                    input = VillainmadEngine.GameInput.Enter },
                new KBI() { key = KeyEsc, button = ButEsc,
                    input = VillainmadEngine.GameInput.Esc },
                new KBI() { key = KeyFire1, button = ButFire1,
                    input = VillainmadEngine.GameInput.Fire1 },
                new KBI() { key = KeyFire2, button = ButFire2,
                    input = VillainmadEngine.GameInput.Fire2 },
                new KBI() { key = KeyFire3, button = ButFire3,
                    input = VillainmadEngine.GameInput.Fire3 },
                new KBI() { key = KeyFocus, button = ButFocus,
                    input = VillainmadEngine.GameInput.Focus },
                new KBI() { key = KeySkip, button = ButSkip,
                    input = VillainmadEngine.GameInput.Skip },
            };
        }

        #region KBI
        

        private static KBI[] _kbiList;
        public static KBI[] KbiList { get { return _kbiList; } }
        #endregion
    }
}
