﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace VillainmadEngine
{
    /// <summary>
    /// Individual entities controlled by the game.
    /// </summary>
    public partial class GameEntity
    {
        /// <summary>
        /// The GameEntity's position.
        /// </summary>
        public Vector2 Position = Vector2.Zero;

        /// <summary>
        /// The GameEntity's velocity/movement.
        /// </summary>
        public Vector2 Velocity = Vector2.Zero;

        private float _angle = 0;
        /// <summary>
        /// The GameEntity's rotated angle, in radians.
        /// </summary>
        public virtual float Angle
        {
            get { return _angle; }
            set { _angle = value % MathHelper.TwoPi; }
        }

        private float _rotation = 0;
        /// <summary>
        /// The speed at which the entity rotates, in radians. If negative, it will rotate
        /// along with its Velocity. (If you want it to rotate 10 degrees backward, just
        /// set it to the radian equivalent of 350 degrees forward.)
        /// </summary>
        public virtual float Rotation
        {
            get { return _rotation; }
            set { _rotation = (value > 0) ? value % MathHelper.TwoPi : value; }
        }

        private Stage _stage;
        /// <summary>
        /// The stage with which the GameEntity is associated.
        /// </summary>
        public Stage Stage
        {
            get { return _stage; }
        }

        #region Constructor
        /// <summary>
        /// Creates a new entity.
        /// </summary>
        public GameEntity()
        {
            _stage = Stage.Active;
            _startFrame = Stage.Frame;
        }
        #endregion
    }
}
