﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VillainmadEngine
{
    public partial class GameEntity
    {

        /// <summary>
        /// The frame at which the entity was created.
        /// </summary>
        public int StartFrame
        {
            get { return _startFrame; }
        }

        private int _startFrame;

        /// <summary>
        /// The number of frames the entity has existed.
        /// </summary>
        public int Age
        {
            get { return DataStealer.ActiveStage.Frame - StartFrame; }
        }

        /// <summary>
        /// Called during Stage.ResetFrame() to ensure the start-frame gets updated. 
        /// </summary>
        public void CheckFrame()
        {
            if (Stage.Active.Frame != DataStealer.OldFrame)
                _startFrame -= DataStealer.OldFrame;
        }

        /// <summary>
        /// True if the entity exists and is currently in the entity-list;
        /// false otherwise.
        /// </summary>
        public bool Extant
        {
            get { return _extant && Stage.Extant(this); }
        }

        /// <summary>
        /// True if the entity has been deleted; false otherwise.
        /// (Warning: this property doesn't actually check if the entity is in the list
        /// of active entities!)
        /// </summary>
        public bool Deleted
        {
            get { return !_extant; }
        }

        private bool _extant = true;

        /// <summary>
        /// Removes the given entity from play.
        /// </summary>
        public void Remove()
        {
            if (!_extant) return;
            _extant = false;
            Stage.RemoveEntity(this);
            OnRemove();
        }

        /// <summary>
        /// Called when the object is removed. Default does nothing.
        /// </summary>
        protected virtual void OnRemove()
        {
        }
    }
}
