﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace VillainmadEngine
{
    public partial class GameEntity
    {
        /// <summary>
        /// Called automatically when the game entity needs to update itself.
        /// </summary>
        /// <param name="gameTime">Current elapsed gametime.</param>
        public void Update(GameTime gameTime)
        {
            if (Deleted) return;
            UpdateBegin(gameTime);
            if (Deleted) return;

            Position = Movement();
            Rectangle GameArea = Stage.GameArea;

            #region Kill If Out-Of-Bounds
            if (Size >= 0)
            {
                if (Position.X < -Size ||
                    Position.X > GameArea.Width + Size ||
                    Position.Y < -Size ||
                    Position.Y > GameArea.Height + Size)
                {
                    Remove();
                    return;
                }
            }
            #endregion

            #region Rotate
            if (Rotation < 0)
            {
                if (Velocity != Vector2.Zero)
                    Angle = Velocity.Angle();
            }
            else Angle += Rotation;
            #endregion

            UpdateEnd(gameTime);
        }

        /// <summary>
        /// Called at the end of the update-period, before collision-detection.
        /// Will NOT be called if the entity was previously deleted.
        /// Default does nothing.
        /// </summary>
        /// <param name="gameTime">Current elapsed gametime.</param>
        protected virtual void UpdateEnd(GameTime gameTime)
        {
        }

        /// <summary>
        /// Called at the beginning of the update-period, before movement and rotation.
        /// Default does nothing.
        /// </summary>
        /// <param name="gameTime">Current elapsed gametime.</param>
        public virtual void UpdateBegin(GameTime gameTime)
        {
        }

        /// <summary>
        /// Called automatically to determine the object's movement; returns the
        /// new location.
        /// Default returns Pos + Vel.
        /// </summary>
        /// <returns>The object's new location.</returns>
        public virtual Vector2 Movement()
        {
            return Position + Velocity;
        }

        /// <summary>
        /// Called automatically when the game entity needs to draw itself.
        /// Default version draws the object's texture at its position,
        /// with standard features such as rotation applied.
        /// </summary>
        /// <param name="gameTime">Current elapsed gametime.</param>
        public virtual void DrawEntity(GameTime gameTime)
        {
            if (Deleted) return;
            Draw(Color.White, true);
        }

        /// <summary>
        /// Called on the object's creation. Default does nothing.
        /// </summary>
        public virtual void OnStart()
        {
        }
    }
}
