﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace VillainmadEngine
{
    public partial class GameEntity
    {
        /// <summary>
        /// Moves the GameEntity to the specified location over a specified
        /// period of time (in frames), optionally stopping at its destination.
        /// </summary>
        /// <param name="Destination">The location to send it to.</param>
        /// <param name="Delay">The amount of time it should take, in frames.</param>
        /// <param name="Stop">If true, the entity will stop once it reaches its
        /// destination. If false, it will keep going.</param>
        public void MoveTo(Vector2 Destination, float Delay, bool Stop)
        {
            if (!Extant) return;

            Vector2 direction = Destination - Position;
            direction /= Delay;

            this.Velocity = direction;

            if (Stop)
            {
                Stage.Active.EnqueueCommand(Delay, delegate()
                {
                    if (Extant) this.Velocity = Vector2.Zero;
                });
            }
        }

        /// <summary>
        /// Moves the GameEntity to the specified location over a specified
        /// period of time, and stops at its destination.
        /// </summary>
        /// <param name="Destination">The location to send it to.</param>
        /// <param name="Delay">The amount of time it should take, in frames.</param>
        public void MoveTo(Vector2 Destination, float Delay)
        {
            MoveTo(Destination, Delay, true);
        }

        /// <summary>
        /// Moves the GameEntity to the specified location over a specified
        /// period of time (in seconds), stopping at its destination.
        /// </summary>
        /// <param name="Destination">The location to send it to.</param>
        /// <param name="Delay">The amount of time it should take, in seconds.</param>
        public void MoveToSecs(Vector2 Destination, float Delay)
        {
            MoveTo(Destination, Delay * 60, true);
        }

        /// <summary>
        /// Moves the GameEntity to the specified location over a specified
        /// period of time (in seconds), optionally stopping at its destination.
        /// </summary>
        /// <param name="Destination">The location to send it to.</param>
        /// <param name="Delay">The amount of time it should take, in seconds.</param>
        /// <param name="Stop">If true, the entity will stop once it reaches its
        /// destination. If false, it will keep going.</param>
        public void MoveToSecs(Vector2 Destination, float Delay, bool Stop)
        {
            MoveTo(Destination, Delay * 60, Stop);
        }
    }
}
