﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VillainmadEngine
{
    public partial class GameState
        : IDisposable
    {
        private bool _disposed = false;

        /// <summary>
        /// Returns whether the entity has been disposed or not.
        /// </summary>
        public bool IsDisposed { get { return _disposed; } }

        /// <summary>
        /// Disposes of the entity's managed and unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            _Dispose(true);
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Disposes of unmanaged resources, and optionally
        /// disposes of managed resources.
        /// </summary>
        /// <param name="disposing">True to dispose of managed resources,
        /// false otherwise.</param>
        private void _Dispose(bool disposing)
        {
            if (disposing)
            {
                OnExit();
            }

            _disposed = true;
        }

        /// <summary>
        /// Disposes of unmanaged resources, and optionally
        /// disposes of managed resources.
        /// </summary>
        /// <param name="disposing">True to dispose of managed resources,
        /// false otherwise.</param>
        protected virtual void Dispose(bool disposing)
        {
        }

        /// <summary>
        /// Destructor.
        /// </summary>
        ~GameState()
        {
            _Dispose(false);
            Dispose(false);
        }
    }
}
