﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VillainmadEngine
{
    public partial class GameState
    {
        /// <summary>
        /// Start a new gamestate.
        /// </summary>
        /// <typeparam name="StateType">The Type of the GameState</typeparam>
        public static void LoadState<StateType>()
            where StateType : GameState, new()
        {
            LoadState(new StateType());
        }

        /// <summary>
        /// Starts a new gamestate with the given State.
        /// </summary>
        /// <param name="NewState">The new State to call.</param>
        public static void LoadState(GameState NewState)
        {
            if (DataStealer.StateStack.Count != 0)
            {
                GameState oldState = DataStealer.ActiveGameState;
                oldState.OnPause();
            }

            DataStealer.StateStack.Add(new StateMenuList(NewState));

            DataStealer.LoadGameState(NewState);
            NewState.OnEnter();
        }

        /// <summary>
        /// Starts a new game of the default type specified in the
        /// Create&lt;type&gt; method.
        /// </summary>
        public static void LoadState()
        {
            LoadState((GameState)Activator.CreateInstance(DataStealer.DefaultType));
        }

        /// <summary>
        /// Ends the current game-state and returns to the previous one.
        /// </summary>
        public static void EndState()
        {
            StateMenuList getStateMenu = DataStealer.StateStackPop();
            GameState getState = getStateMenu.State;

            while (DataStealer.ActiveMenu != null)
                Menu.Close();

            getStateMenu.MenuList = null;

            if (DataStealer.StateStack.Count == 1)
            {
                Basis.Exit();
                return;
            }

            getState.Dispose();

            getState = DataStealer.ActiveGameState;

            DataStealer.LoadGameState(getState);
            getState.OnResume();
        }
    }
}
