﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VillainmadEngine
{
    /// <summary>
    /// Used by GameState and Stage classes. 
    /// </summary>
    public interface IContentLoader
    {
        /// <summary>
        /// List of filenames of 2D textures. Starts in the "gfx/" directory in
        /// Content. (Not including extensions)
        /// </summary>
        string[] Texture2DList { get; }

        /// <summary>
        /// List of filenames of sound effects. Starts in the "sfx/" directory in
        /// Content. (Not including extensions)
        /// </summary>
        string[] SoundEffectList { get; }

        /// <summary>
        /// If not null, the filename of the Xact Audio Engine (.xgs) file to be loaded.
        /// Starts in the "sfx/" directory in Content. (Not including extension)
        /// </summary>
        string AudioEngine { get; }

        /// <summary>
        /// If not null, the filename of the Xact Sound Bank (.xsb) file to be loaded. 
        /// Starts in the "sfx/" directory in Content. (Not including extension) 
        /// </summary>
        string SoundBank { get; }

        /// <summary>
        /// If not null, the filename of the Xact Wave Bank (.xwb) file to be loaded. 
        /// Starts in the "sfx/" directory in Content. (Not including extension) 
        /// </summary>
        string WaveBank { get; }

        /// <summary>
        /// Called after the object has been constructed, but before the content 
        /// begins loading.
        /// </summary>
        void LoadBegin();

        /// <summary>
        /// Called after the content is finished loading but before any updates have
        /// been called or OnEnter() has been called.
        /// </summary>
        void LoadEnd();
    }
}
