﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VillainmadEngine
{
    /// <summary>
    /// Base class for Stages and Menus.
    /// </summary>
    public partial class INCommandUser
        : INDrawUpdateInput
    {
        private List<QueuedCommand> CommandList = new List<QueuedCommand>();

        /// <summary>
        /// Enqueues a command with a particular delay.
        /// </summary>
        /// <param name="SecondDelay">The number of seconds to delay the 
        /// command (can be zero or negative, in which case it will be called
        /// immediately).</param>
        /// <param name="Command">The command to perform, in the form of
        /// delegate(){ &lt;action-list&gt; }</param>
        public QueuedCommand EnqueueCommandSecs(float SecondDelay, Action Command)
        {
            return EnqueueCommand((int)(SecondDelay * 60F), Command);
        }

        /// <summary>
        /// Enqueues a command with a particular delay.
        /// </summary>
        /// <param name="FrameDelay">The number of frames to delay the 
        /// command (can be zero or negative, in which case it will be called
        /// immediately).</param>
        /// <param name="Command">The command to perform, in the form of
        /// delegate(){ &lt;action-list&gt; }</param>
        public QueuedCommand EnqueueCommand(float FrameDelay, Action Command)
        {
            int Delay = Frame + (int)Math.Round(FrameDelay);

            QueuedCommand theCommand = new QueuedCommand(Delay, Command);
            CommandList.Add(theCommand);

            return theCommand;
        }
    }
}