﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VillainmadEngine
{
    public partial class INDrawUpdateInput
    {
        /// <summary>
        /// Called automatically to enact input.
        /// </summary>
        public void Input()
        {
            Input(CurrentInput.InputList, CurrentInput.InputPressed,
                CurrentInput.InputReleased);
        }

        /// <summary>
        /// <para>Called automatically to enact input. Default form does nothing.</para>
        /// <para>You do it like this: each of the arguments are flag-enums of 
        /// the GameInput type. To determine if a given button is currently down/has
        /// just been pressed/has just been released, use a boolean comparison to
        /// the list.</para>
        /// <para>For instance, if you want to check if the Fire1 button has just been
        /// pressed, you would do an if-statement in the form of "if((InputPressed &amp;
        /// GameInput.Fire1) != 0) ..." or "if((InputPressed &amp; GameInput.Fire1) == 
        /// GameInput.Fire1) ..."</para>
        /// </summary>
        /// <param name="InputList">List of buttons etc. which are currently pressed
        /// down.</param>
        /// <param name="InputPressed">List of buttons etc. which have just been 
        /// pressed and were not previously down.</param>
        /// <param name="InputReleased">List of buttons etc. which have just been 
        /// released and are no longer down.</param>
        protected virtual void Input(GameInput InputList, GameInput InputPressed,
            GameInput InputReleased)
        {
        }
    }
}
