﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VillainmadEngine
{
    public partial class Menu
    {
        /// <summary>
        /// <para>Called automatically to enact input.</para>
        /// <para>You do it like this: each of the arguments are flag-enums of 
        /// the GameInput type. To determine if a given button is currently down/has
        /// just been pressed/has just been released, use a boolean comparison to
        /// the list.</para>
        /// <para>For instance, if you want to check if the Fire1 button has just been
        /// pressed, you would do an if-statement in the form of "if((InputPressed &amp;
        /// GameInput.Fire1) != 0) ..." or "if((InputPressed &amp; GameInput.Fire1) == 
        /// GameInput.Fire1) ..."</para>
        /// </summary>
        /// <param name="InputList">List of buttons etc. which are currently pressed
        /// down.</param>
        /// <param name="InputPressed">List of buttons etc. which have just been 
        /// pressed and were not previously down.</param>
        /// <param name="InputReleased">List of buttons etc. which have just been 
        /// released and are no longer down.</param>
        protected override void Input(GameInput InputList, GameInput InputPressed,
            GameInput InputReleased)
        {
            if ((InputPressed & GameInput.DDown) != 0)
            {
                Selected++;
            }
            if ((InputPressed & GameInput.DUp) != 0)
            {
                Selected--;
            }

            if ((InputPressed & (GameInput.Fire1 | GameInput.Enter)) != 0)
            {
                CurrentItem.Select();
            }
            if ((InputPressed & GameInput.Esc) == GameInput.Esc
                || (InputPressed & GameInput.Fire2) == GameInput.Fire2)
            {
                Close();
            }
        }
    }
}
