﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VillainmadEngine
{
    public partial class Menu
    {
        /// <summary>
        /// Calls a new menu of the specified type and adds it to the list of menus.
        /// </summary>
        /// <typeparam name="MenuType">The new type of the menu.</typeparam>
        public static void StartMenu<MenuType>()
            where MenuType : Menu, new()
        {
            StartMenu(new MenuType());
        }

        /// <summary>
        /// Adds the menu to the list of currently-active menus.
        /// </summary>
        /// <param name="NewMenu">The new menu.</param>
        public static void StartMenu(Menu NewMenu)
        {
            if (DataStealer.StateStackPeek.MenuList.Count != 0)
            {
                DataStealer.StateStackPeek.MenuPeek.OnPause();
            }
            else
            {
                if (DataStealer.ActiveStage != null) DataStealer.ActiveStage.OnPause();
                DataStealer.ActiveGameState.OnPause();
            }
            
            DataStealer.StateStackPeek.MenuList.Add(NewMenu);

            NewMenu.OnEnter();
        }

        /// <summary>
        /// Ends currently-active menu.
        /// </summary>
        public static void Close()
        {
            Menu oldMenu = DataStealer.StateStackPeek.MenuPop();

            oldMenu.OnExit();
            oldMenu.Dispose();

            if (DataStealer.StateStackPeek.MenuList.Count != 0)
            {
                DataStealer.StateStackPeek.MenuPeek.OnResume();
            }
            else
            {
                if (DataStealer.ActiveStage != null) DataStealer.ActiveStage.OnResume(); 
                DataStealer.ActiveGameState.OnResume();
            }
        }
    }
}
