﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;

namespace VillainmadEngine
{
    public static partial class Sound
    {
        private static List<NamedInstance> SoundList = new List<NamedInstance>(15);

        /// <summary>
        /// Plays a sound effect as a SoundEffectInstance.
        /// You can only have up to 16 different SoundEffectInstances
        /// playing at once, including the BGM; this may result in sounds being cut off, 
        /// depending on the number of sounds playing simultaneously.
        /// </summary>
        /// <param name="Filename">Filename of the sound in the "sfx/" folder in 
        /// Content. Does not include extension.</param>
        public static void PlaySound(string Filename)
        {
            PlaySound(Filename, 1F, 0F, 0F);
        }

        /// <summary>
        /// Plays a sound effect as a SoundEffectInstance.
        /// You can only have up to 16 different SoundEffectInstances
        /// playing at once, including the BGM; this may result in sounds being cut off, 
        /// depending on the number of sounds playing simultaneously.
        /// </summary>
        /// <param name="Filename">Filename of the sound in the "sfx/" folder in 
        /// Content. Does not include extension.</param>
        /// <param name="Volume">Volume, ranging from 0.0F (silence) to 1.0F (full
        /// volume).</param>
        /// <param name="Pitch">Pitch adjustment, ranging from -1.0F (down one ocative)
        /// to 1.0F (up one ocative). 0.0F is normal pitch.</param>
        /// <param name="Pan">Panning, ranging from -1.0F (full left) to 1.0F (full
        /// right). 0.0F is centered.</param>
        public static void PlaySound(string Filename, float Volume,
            float Pitch, float Pan)
        {
            SoundEffectCheck(Filename);

            if (SoundList.Count == 15) RemoveInstanceZero();
            
            SoundEffectInstance NewSound =
                DataStealer.SoundEffectList[Filename].CreateInstance();
            
            NewSound.Pan = Pan;
            NewSound.Pitch = Pitch;
            NewSound.Volume = Volume;
            NewSound.Play();
            SoundList.Add(new NamedInstance(NewSound, Filename));
        }

        private static void RemoveInstanceZero()
        {
            RemoveInstance(0);
        }

        private static void RemoveInstance(int i)
        {
            SoundList[i].Instance.Stop();
            SoundList[i].Dispose();
            SoundList.RemoveAt(i);
        }

        private static void RemoveInstance(string Filename)
        {
            for (int i = 0; i < SoundList.Count; i++)
            {
                while (i < SoundList.Count &&
                    SoundList[i].Name == Filename)
                    RemoveInstance(i);
            }
        }
    }
}
