﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;

namespace VillainmadEngine
{
    public static partial class Sound
    {
        private struct NamedInstance : IDisposable
        {
            public string Name;
            public SoundEffectInstance Instance;

            public SoundState State
            {
                get { return Instance.State; }
            }

            public void Dispose()
            {
                Instance.Stop();
                Instance.Dispose();
                Name = null;
            }

            public bool IsDisposed
            {
                get { return Instance.IsDisposed; }
            }

            public bool IsLooped
            {
                get { return Instance.IsLooped; }
                set { Instance.IsLooped = value; }
            }

            public void Play()
            {
                Instance.Play();
            }

            public void Stop()
            {
                Instance.Stop();
            }

            public void Pause()
            {
                Instance.Pause();
            }

            public void Resume()
            {
                Instance.Resume();
            }

            public NamedInstance(SoundEffect Effect)
                : this(Effect.CreateInstance(), Effect.Name)
            {
            }

            public NamedInstance(SoundEffectInstance Instance, string Name)
            {
                if (Name == null || Name == "" || Name.Length == 0)
                    throw new ArgumentNullException("Instance must have a name!");

                this.Name = Name;
                this.Instance = Instance;
            }

            public static explicit operator NamedInstance(SoundEffect effect)
            {
                return new NamedInstance(effect);
            }
        }
    }
}