﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace VillainmadEngine
{
    public partial class Stage
    {
        /// <summary>
        /// Called automatically when the stage needs to update itself.
        /// </summary>
        /// <param name="gameTime">Current elapsed gametime.</param>
        public sealed override void Update(GameTime gameTime)
        {
            DoCommands();
            if (Player != null && Player.Extant)
                Player.Input();

            for (int i = 0; i < EntityArray.Length; i++)
            {
                for (int j = 0; j < EntityArray[i].Count; j++)
                {
                    GameEntity curEnt;
                    do
                    {
                        curEnt = EntityArray[i][j];
                        curEnt.Update(gameTime);
                    }
                    while (j < EntityArray[i].Count && curEnt.Deleted);
                }
            }
            #region Keep Player In Bounds
            if (!PlayerField.IsEmpty && Player != null && Player.Extant)
            {

                if (Player.Position.X < PlayerField.X) Player.Position.X = PlayerField.X;
                if (Player.Position.Y < PlayerField.Y) Player.Position.Y = PlayerField.Y;
                if (Player.Position.X > PlayerField.X + PlayerField.Width)
                    Player.Position.X = PlayerField.X + PlayerField.Width;
                if (Player.Position.Y > PlayerField.Y + PlayerField.Height)
                    Player.Position.Y = PlayerField.Y + PlayerField.Height;
            }
            #endregion

            //TODO: Collision Detection
        }

        /// <summary>
        /// Called whenever the stage needs to draw itself.
        /// </summary>
        /// <param name="gameTime">Current elapsed gametime.</param>
        public void Draw(GameTime gameTime)
        {
            DataStealer.SpriteBatch.End();
            DataStealer.GraphicsDevice.Viewport = _viewport;
            DataStealer.blendState = Drawing.DefaultBlend;
            DataStealer.StartBatch();

            DrawBackground(gameTime);

            for (int i = 0; i < EntityArray.Length; i++)
            {
                for (int j = 0; j < EntityArray[i].Count; j++)
                    EntityArray[i][j].DrawEntity(gameTime);
            }

            DrawForeground(gameTime);

            DataStealer.SpriteBatch.End();
            DataStealer.GraphicsDevice.Viewport = DataStealer.DefaultViewport;
            DataStealer.blendState = Drawing.DefaultBlend;
            DataStealer.StartBatch();
        }
    }
}
