﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace VillainmadEngine
{
    public partial class Stage
    {
        /// <summary>
        /// Resets the frames back to zero, updating enqueued commands as necessary.
        /// </summary>
        public new void ResetFrame()
        {
            DataStealer.OldFrame = Frame;
            for (int i = 0; i < EntityArray.Length; i++)
            {
                for (int j = 0; j < EntityArray[i].Count; j++)
                {
                    EntityArray[i][j].CheckFrame();
                }
            }
            base.ResetFrame();
            DataStealer.OldFrame = -1;
        }

        /// <summary>
        /// Same as Stage.Frame, except during Stage.Reset() when Stage.Frame 
        /// is being ... reset.
        /// </summary>
        public int PrevFrame
        {
            get
            {
                if (DataStealer.OldFrame == -1) return Frame;
                return DataStealer.OldFrame;
            }
        }

        /// <summary>
        /// The number of entity-layers.
        /// </summary>
        public virtual int NumLayers
        {
            get { throw new NotImplementedException("Need to specify number of layers!"); }
        }

        /// <summary>
        /// Array of the lists of entities!
        /// </summary>
        private List<GameEntity>[] EntityArray;

        /// <summary>
        /// Collection of every entity in the stage, arranged by layer.
        /// </summary>
        public GameEntity[][] Entities
        {
            get
            {
                GameEntity[][] ReturnArray = new GameEntity[EntityArray.Length][];
                for (int i = 0; i < EntityArray.Length; i++)
                    ReturnArray[i] = EntityArray[i].ToArray();
                
                return ReturnArray;
            }
        }
    }
}
