﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace VillainmadEngine
{
    public partial class Stage
    {
        private PlayerCharacter _player = null;

        /// <summary>
        /// The starting position of the player.
        /// </summary>
        public virtual Vector2 PlayerStart
        {
            get { throw new NotImplementedException("Need to specify player-start!"); }
        }

        /// <summary>
        /// The current object representing the player-character.
        /// </summary>
        public PlayerCharacter Player
        {
            get { return _player; }
        }

        /// <summary>
        /// Adds a player to the given position, despawning the existing player
        /// if necessary.
        /// </summary>
        /// <typeparam name="pType">Object-type of the player character.</typeparam>
        public void AddPlayer<pType>()
            where pType : PlayerCharacter, new()
        {
            RemPlayer();

            _player = (PlayerCharacter)AddEntity<pType>(PlayerStart, Vector2.Zero);
        }

        /// <summary>
        /// Despawns the player.
        /// </summary>
        public void RemPlayer()
        {
            if (_player == null || !_player.Extant) return;
            Player.Remove();
            _player = null;
        }
    }
}
