﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VillainmadEngine
{
    public partial class Stage
    {
        /// <summary>
        /// Starts a new stage of the given type, ending the old one if any.
        /// </summary>
        /// <typeparam name="stype">Type of the stage to load.</typeparam>
        public static void LoadStage<stype>()
            where stype : Stage, new()
        {
            LoadStage(new stype());
        }
        
        /// <summary>
        /// Starts the new given stage, ending the old one if any.
        /// </summary>
        /// <param name="newStage">The stage to load.</param>
        public static void LoadStage(Stage newStage)
        {
            while (Menu.Active != null)
            {
                Menu.Close();
            }
            if (DataStealer.ActiveStage != null)
            {
                DataStealer.ActiveStage.OnExit();
                DataStealer.ActiveStage.Dispose();
                DataStealer.StateStackPeek.Stage = null;
            }
            DataStealer.StateStackPeek.Stage = newStage;

            newStage.LoadBegin();
            DataStealer.LoadStage(newStage);
            newStage.LoadEnd();
            newStage.OnEnter();
        }

        /// <summary>
        /// Ends the currently-active stage.
        /// </summary>
        public static void EndStage()
        {
            if (DataStealer.StateStackPeek.Stage == null) return;
            Stage endingStage = DataStealer.StateStackPeek.Stage;

            endingStage.OnExit();
            endingStage.Dispose();

            DataStealer.Texture2DStage = new string[] { };
            DataStealer.SoundEffectStage = new string[] { };
            DataStealer.SoundBankStage = null;
            DataStealer.WaveBankStage = null;
            DataStealer.AudioEngineStage = null;

            DataStealer.UnloadUnnecessary();
        }
    }
}
