﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Input;
using Nuclex.Input;
using Nuclex.Input.Devices;

namespace VillainmadEngine
{
    /// <summary>
    /// Information on the player input.
    /// </summary>
    internal static partial class CurrentInput
    {
        static CurrentInput()
        {
            input = new InputManager();

            pad = input.GetGamePad(ExtendedPlayerIndex.Five);

            directions = new Direction[4];
            directions[0] = new Direction(Keys.Up, Buttons.DPadUp,
                GameInput.DUp,
                false, Direction.XY.Y, Keys.Down, Buttons.DPadDown);
            directions[1] = new Direction(Keys.Right, Buttons.DPadRight,
                GameInput.DRight, false, Direction.XY.X, Keys.Left,
                Buttons.DPadLeft);
            directions[2] = new Direction(Keys.Down, Buttons.DPadDown,
                GameInput.DDown, directions[0]);
            directions[3] = new Direction(Keys.Left, Buttons.DPadLeft,
                GameInput.DLeft,
                directions[1]);

            DataStealer.Basis.Components.Add(input);
        }

        private static InputManager input;
        private static IGamePad pad;
        private static ExtendedGamePadState diState;
        private static GamePadState dxState;
        private static KeyboardState kbState;
        private static LocalizedKeyboardState keyboardState;

        private static GameInput _inputList = 0;
        private static GameInput _inputPressed = 0;
        private static GameInput _inputReleased = 0;

        /// <summary>
        /// Returns the list of current input keys.
        /// </summary>
        public static GameInput InputList
        {
            get { return _inputList; }
        }

        /// <summary>
        /// Returns the list of keys which have just been pressed.
        /// </summary>
        public static GameInput InputPressed
        {
            get { return _inputPressed; }
        }

        /// <summary>
        /// Returns the list of keys which have just been released.
        /// </summary>
        public static GameInput InputReleased
        {
            get { return _inputReleased; }
        }

        /// <summary>
        /// Gets the currently-input keys, and returns a list of the GameInput
        /// values.
        /// </summary>
        public static void GetInput()
        {
            _inputPressed = 0;
            _inputReleased = 0;

            diState = pad.GetExtendedState();
            dxState = pad.GetState();
            kbState = Keyboard.GetState();
            keyboardState = new LocalizedKeyboardState(kbState);

            foreach (Configuration.KBI curKBI in Configuration.Config.KbiList)
            {

                if (keyboardState.IsKeyDown(curKBI.key)
                    || diState.IsButtonDown(curKBI.button))
                {
                    if ((_inputList & curKBI.input) == 0)
                    {
                        _inputList = _inputList | curKBI.input;
                        _inputPressed = _inputPressed | curKBI.input;
                    }
                }
                else if ((_inputList & curKBI.input) == curKBI.input)
                {
                    _inputList = _inputList ^ curKBI.input;
                    _inputReleased = _inputReleased | curKBI.input;
                }
            }

            foreach (Direction curDir in directions)
            {
                if (curDir.IsDown(keyboardState, dxState))
                {
                    if ((_inputList & curDir.Input) == 0)
                    {
                        _inputList = _inputList | curDir.Input;
                        _inputPressed = _inputPressed | curDir.Input;
                    }
                }
                else if ((_inputList & curDir.Input) != 0)
                {
                    _inputList = _inputList ^ curDir.Input;
                    _inputReleased = _inputReleased | curDir.Input;
                }
            }
        }
    }
}
