﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Input;
using Nuclex.Input;
using Nuclex.Input.Devices;

namespace VillainmadEngine
{
    internal static partial class CurrentInput
    {
        private struct Direction
        {
            #region Values
            public enum XY { X, Y }

            public Keys Key;
            public Keys OppositeKey;
            public Buttons Button;
            public Buttons OppositeButton;
            public GameInput Input;
            public bool Negative;
            public XY XorY;
            #endregion

            #region Constructors
            public Direction(Keys key, Buttons button, GameInput input,
                bool negative, XY XorY, Keys oppositeKey, Buttons oppositeButton)
            {
                this.Key = key;
                this.Button = button;
                this.XorY = XorY;
                this.Input = input;
                this.Negative = negative;
                this.OppositeButton = oppositeButton;
                this.OppositeKey = oppositeKey;
            }

            public Direction(Keys key, Buttons button, GameInput input, Direction
                opposite)
                : this(key, button, input, !opposite.Negative, opposite.XorY,
                opposite.Key, opposite.Button)
            {

            }
            #endregion

            #region IsDown
            public bool SafeIsDown(LocalizedKeyboardState kbState,
                GamePadState diState)
            {
                if (kbState.IsKeyDown(this.Key)
                    //|| diState.IsButtonDown(this.Button)
                    )
                    return true;

                if (XorY == XY.X) return (Negative) ?
                     diState.ThumbSticks.Left.X <= -Configuration.Config.Deadzone :
                     diState.ThumbSticks.Left.X >= Configuration.Config.Deadzone;
                return (Negative) ?
                    diState.ThumbSticks.Left.Y <= -Configuration.Config.Deadzone :
                    diState.ThumbSticks.Left.Y >= Configuration.Config.Deadzone;
            }

            public bool IsDown(LocalizedKeyboardState kbState,
                GamePadState diState)
            {
                if (kbState.IsKeyDown(this.OppositeKey)
                     || diState.IsButtonDown(this.OppositeButton)
                    ) return false;
                if (XorY == XY.X)
                {
                    if ((Negative) ?
                        diState.ThumbSticks.Left.X >= Configuration.Config.Deadzone :
                        diState.ThumbSticks.Left.X <= -Configuration.Config.Deadzone) return false;
                }
                else if ((Negative) ?
                    diState.ThumbSticks.Left.Y >= Configuration.Config.Deadzone :
                    diState.ThumbSticks.Left.Y <= -Configuration.Config.Deadzone) return false;

                return SafeIsDown(kbState, diState);
            }
            #endregion
        }

        private static Direction[] directions;
    }
}
